#####
###  PropertyFolder - a generic PropertyFolder for ZOPE
##   Copyright (C) 2003 Maik Jablonski (maik.jablonski@uni-bielefeld.de)
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
##   along with this program; if not, write to the Free Software
###  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#####

from urllib import quote
from Globals import DTMLFile, InitializeClass
from OFS.Folder import Folder
from OFS.PropertyManager import PropertyManager
from Products.ZCatalog.CatalogPathAwareness import CatalogAware

def manage_addPropertyFolder (self, id, title = '', REQUEST = None):
    """ Add a new PropertyFolder to current ObjectManager """
    
    ob = PropertyFolder(id, title, REQUEST)
    self._setObject(id, ob)
    
    if REQUEST:
        try: u = self.DestinationURL()
        except: u = REQUEST['URL1']
        if REQUEST.get('submit') == " Add and Edit ": u = "%s/%s" % (u, quote(id))
        REQUEST.RESPONSE.redirect(u + '/manage_main')

manage_addPropertyFolderForm = DTMLFile('dtml/addForm', globals())

class PropertyFolder(CatalogAware, Folder):
    """ PropertyFolder: Just a simple PropertyFolder with locks """
   
    meta_type='PropertyFolder'
    
    id = title = object_type = PropertyFolderDescription = ''
    
    possible_meta_types=Folder.all_meta_types
    selected_meta_types=[]
    
    property_extensible_schema__ = 1
    published = 0

    __ac_permissions__ = Folder.__ac_permissions__+(
        ('View',('__call__', 'text_content', 'PrincipiaSearchSource')),
        ('PropertyFolder: Edit', ('manage_propertiesForm',
                     'manage_editProperties',
                     'manage_changeProperties',
                     'manage_addProperty',
                     'manage_delProperties',
                     'manage_movePropertyUp',
                     'manage_movePropertyDown')
                    ),
        ('PropertyFolder: Manage',
        ('manage_manageForm', 'manage_PropertyFolder')),
        ('PropertyFolder: Publish',
        ('manage_publish', 'manage_unpublish')),
        )
    
    _properties = ({'id':'title', 'type':'string', 'mode':'w'}, )
    
    manage_options = Folder.manage_options + (
                     {'label': 'Manage',  'action': 'manage_manageForm'},)
        
    manage_manageForm = DTMLFile('dtml/manageForm', globals())
    manage_propertiesForm = DTMLFile('dtml/propertiesForm', globals())


    def __init__(self,id,title='',REQUEST=None):
        """ Set id & title of PropertyFolder """

        self.id = id
        self.title = title
        self.object_type = ''
        self.property_extensible_schema__ = 1
        self.published = 0
        self.PropertyFolderDescription = ''
        self.selected_meta_types = []
        self.reindex_object()


    def all_meta_types(self):
        ''' Return list with selected meta_types. '''
        
        mt = []
       
        for meta_type in self.possible_meta_types():
            if meta_type['name'] in self.selected_meta_types:
                mt.append(meta_type)

        if not mt:
            mt = self.possible_meta_types()

        return mt


    def manage_PropertyFolder(self,
                              object_type = '', 
                              default_catalog = 'Catalog', 
                              property_extensible_schema__ = 0, 
                              PropertyFolderDescription = '', 
                              selected_meta_types = [],
                              update = '', 
                              REQUEST = None):
        """ Manages a PropertyFolder """
        
        old_object_type = self.object_type
    
        self.object_type = object_type
        self.default_catalog = default_catalog
        self.property_extensible_schema__ = property_extensible_schema__
        self.PropertyFolderDescription = PropertyFolderDescription
        self.selected_meta_types = selected_meta_types
        self.reindex_object()
        
        if update:
            # Go up to AbracadabaObject's standard_object_folder if exists and
            # search PropertyObjects with same object_type in all subfolders
            
            if hasattr(self,'standard_object_folder'):
                startFrom = getattr(self,
                                    'standard_object_folder').aq_inner.aq_parent
            else:
                startFrom = self.aq_parent

            resultObjects = startFrom.ZopeFind(
                         startFrom,
                         obj_metatypes = [self.meta_type],
                         obj_expr = """
                                     object_type=='%s' and
                                     _.SecurityCheckPermission(
                                            'PropertyFolder: Manage', this())
                                    """ % (old_object_type),
                         search_sub = 1
                        )

            selfMap = self.propertyMap()

            # iterate through the results
            for (id,obj) in resultObjects:
                # set the manage-vars in the client
                obj.object_type = self.object_type
                obj.default_catalog = self.default_catalog
                obj.property_extensible_schema__ = self.property_extensible_schema__
                obj.PropertyFolderDescription = self.PropertyFolderDescription
                obj.selected_meta_types = self.selected_meta_types
                
                # add new properties to clients
                for propertyTuple in selfMap:
                    id = propertyTuple['id']
                    if not obj.hasProperty(id):
                        value = propertyTuple.get('select_variable',
                                                        self.getProperty(id))

                        if not value:
                            value = self.getProperty(id)
                            
                        obj.manage_addProperty(id, value, propertyTuple['type'])
                        # Sync values for (multiple)-selections
                        obj.manage_changeProperties({id: self.getProperty(id)})
                
                # delete old or additional client-properties
                clientMap = obj.propertyMap()
                for propertyTuple in clientMap:
                    id = propertyTuple['id']
                    if not self.hasProperty(id):
                        obj.manage_delProperties([id])
                        
                # And now sync the whole dict for the ordering                          
                obj._properties = self._properties
            
        if REQUEST:
            return self.manage_manageForm(self,
                                          REQUEST,
                                          manage_tabs_message = 
                                                'Your changes have been saved',
                                          update_menu = 1)

    def manage_publish(self, REQUEST=None):
        """ Set published-state to 1. """
        
        self.published = 1
        self.reindex_object()
                
        if REQUEST:
            return self.manage_propertiesForm(self, REQUEST, 
                                    manage_tabs_message = 
                                            "Published %s" % self.meta_type,
                                    update_menu = 1)

    def manage_unpublish(self, REQUEST=None):
        """ Set published-state to 0. """
        
        self.published = 0
        self.reindex_object()
        
        if REQUEST:
            return self.manage_propertiesForm(self, REQUEST, 
                                    manage_tabs_message = 
                                            "Unpublished %s" % self.meta_type,
                                    update_menu = 1)

    def manage_addProperty(self, id, value, type, REQUEST=None):
        """ Add property and reindex. """
        
        PropertyManager.manage_addProperty(self, id, value, type, REQUEST)
        self.reindex_object()
        if REQUEST:
            return self.manage_propertiesForm(self, REQUEST,
                                              manage_tabs_message="Saved changes.")
        
    def manage_delProperties(self, ids=None, REQUEST=None):
        """ Delete properties and reindex. """
        
        PropertyManager.manage_delProperties(self, ids, REQUEST)
        self.reindex_object()
        if REQUEST:
            return self.manage_propertiesForm(self, REQUEST,
                                              manage_tabs_message="Saved changes.")
                                              
    def manage_changeProperties(self, REQUEST=None, **kw):
        """ Update properties and reindex. """
        
        PropertyManager.manage_changeProperties(self, REQUEST, **kw)
        self.reindex_object()
        if REQUEST:
            return self.manage_propertiesForm(self, REQUEST,
                                              manage_tabs_message="Saved changes.")

    def manage_editProperties(self, REQUEST=None):
        """ Edit Properties and reindex. """
        
        PropertyManager.manage_editProperties(self, REQUEST)
        self.reindex_object()
        if REQUEST:
            return self.manage_propertiesForm(self, REQUEST,
                                              manage_tabs_message="Saved changes.")
    
    def manage_movePropertyUp(self, id, REQUEST=None):
        """ Moves property with id one position up. """
        
        properties = self._properties
        for i in range(0, len(properties)):
            if properties[i]['id']==id: 
                index = i
                if index > 0:
                    prevProperty = properties[index-1]
                    currProperty = properties[index]
                    self._properties=properties[:index-1] + (currProperty,
                                         prevProperty) + properties[index+1:]

        if REQUEST: return self.manage_propertiesForm(self, REQUEST, update_menu=1)
                                
    def manage_movePropertyDown(self,id,REQUEST=None):
        """  Moves property with id one position down.  """
        
        properties = self._properties
        for i in range(0, len(properties)):
            if properties[i]['id'] == id:
                index = i
                if index + 1 < len(properties):
                    currProperty = properties[index]
                    nextProperty = properties[index+1]
                    self._properties = properties[:index] + (nextProperty, 
                                          currProperty) + properties[index+2:]

        if REQUEST: return self.manage_propertiesForm(self,REQUEST,update_menu=1)
  
    def _get_browser_default_(self):
        """ Returns the rendering-method as object. """
        
        if hasattr(self, self.object_type) and self.object_type <> self.id:
            return getattr(self, self.object_type)
        else:
            return self.index_html
        
    def __browser_default__(self, REQUEST=None):
        """ Hook into Zope-BaseRequest; returns self and rendering-method-id. """

        # Return collection for WebDav...
        if REQUEST.get('REQUEST_METHOD')=='PROPFIND':
            return (self, ['PROPFIND'])

        # else return our custom browser_default.
        else:
            return (self, [self._get_browser_default_().getId()])

    def __call__(self, REQUEST=None, RESPONSE=None, **kw):
        """ Render the page with specified object_type or index_html. """
        
        if REQUEST is None: REQUEST = self.REQUEST
        if RESPONSE is None: RESPONSE = REQUEST.RESPONSE

        return self._get_browser_default_()(self, REQUEST, RESPONSE, **kw)
    
InitializeClass(PropertyFolder)
